package com.atlassian.confluence.jspwiki;

import com.atlassian.confluence.core.EntityObject;
import com.atlassian.confluence.importexport.ImportExportException;
import com.atlassian.confluence.importexport.ImportExportManager;
import com.atlassian.confluence.importexport.impl.BackupExporter;
import com.atlassian.confluence.pages.Attachment;
import com.atlassian.confluence.pages.Page;
import com.atlassian.confluence.security.SpacePermission;
import com.atlassian.confluence.spaces.Space;
import com.atlassian.confluence.spaces.SpaceDescription;
import com.atlassian.confluence.util.FileUtils;
import com.ecyrd.jspwiki.PageManager;
import com.ecyrd.jspwiki.WikiPage;
import com.ecyrd.jspwiki.attachment.AttachmentManager;
import com.ecyrd.jspwiki.providers.ProviderException;
import com.ecyrd.jspwiki.providers.WikiPageProvider;

import java.io.File;
import java.io.InputStream;
import java.util.*;
import java.sql.Timestamp;

public class JspWikiBackupExporter extends BackupExporter
{
    private ContentCleaner contentCleaner;
    private AttachmentManager attachmentManager;
    private PageManager pageManager;
    private long next = 1;
    private String spaceName;
    private String creatorUserName;
    private Space space;
    private Map pagesMap = new HashMap();
    private List attachments = new ArrayList();
    private File baseExportPath;

    public JspWikiBackupExporter(PageManager pageManager, AttachmentManager attachmentManager, String spaceName, String creatorUserName, Properties properties)
    {
        this.pageManager = pageManager;
        this.attachmentManager = attachmentManager;
        this.spaceName = spaceName;
        this.creatorUserName = creatorUserName;

        Properties variables = new Properties(properties);
        variables.setProperty(ContentCleaner.VAR_JSPWIKI_VERSION, "2.0.52");//?
        variables.setProperty(ContentCleaner.VAR_PAGE_NAME, "This page");//haha good one!
        variables.setProperty(ContentCleaner.VAR_TOTAL_PAGES, String.valueOf(pageManager.getTotalPageCount()));
        this.contentCleaner = new ContentCleaner(variables);
    }

    protected String getExportType() throws ImportExportException
    {
        return ImportExportManager.EXOPRT_XML_TYPE_SPACE;
    }

    protected List getObjectsForExport() throws ImportExportException
    {
        ArrayList objects = new ArrayList();
        addSpaceForExport(objects);
        addPagesForExport();
        addAttachmentsForExport();

        return objects;
    }

    private void addSpaceForExport(ArrayList objects)
    {
        space = new Space();
        space.setKey(spaceName);
        space.setName(spaceName);
        setCreationModificationInfo(space);
        SpaceDescription description = new SpaceDescription();
        setCreationModificationInfo(description);
        description.setContent(spaceName);
        space.setDescription(description);
        description.setSpace(space);

        //initially usable by the creator only
        SpacePermission perm1 = new SpacePermission(SpacePermission.VIEWSPACE_PERMISSION, space, null, creatorUserName);
        setCreationModificationInfo(perm1);
        space.addPermission(perm1);
        SpacePermission perm2 = new SpacePermission(SpacePermission.EDITSPACE_PERMISSION, space, null, creatorUserName);
        setCreationModificationInfo(perm2);
        space.addPermission(perm2);
        SpacePermission perm3 = new SpacePermission(SpacePermission.COMMENT_PERMISSION, space, null, creatorUserName);
        setCreationModificationInfo(perm3);
        space.addPermission(perm3);
        SpacePermission perm4 = new SpacePermission(SpacePermission.SETSPACEPERMISSIONS_PERMISSION, space, null, creatorUserName);
        setCreationModificationInfo(perm4);
        space.addPermission(perm4);

        objects.add(space);
    }

    private void setCreationModificationInfo(EntityObject entity)
    {
        entity.setId(next++);
        entity.setCreationDate(new Timestamp(new Date().getTime()));
        entity.setCreatorName(creatorUserName);
        entity.setLastModificationDate(new Timestamp(new Date().getTime()));
        entity.setLastModifierName(creatorUserName);
    }

    private void addPagesForExport() throws ImportExportException
    {
        Collection allPages = null;

        try
        {
            allPages = pageManager.getAllPages();
        }
        catch (ProviderException e)
        {
            throw new ImportExportException(e);
        }

        for (Iterator allPagesIter = allPages.iterator(); allPagesIter.hasNext();)
        {
            WikiPage wikiPage = (WikiPage) allPagesIter.next();
            if (unExportableWikiPage(wikiPage))
            {
                continue;
            }

            try
            {
                Page page = createPage(wikiPage);
                space.addPage(page);
                pagesMap.put(page.getTitle(), page);
            }
            catch (Exception e)
            {
                e.printStackTrace();
                System.out.println("exporting page failed: " + wikiPage.getName());
            }
        }
    }

    private void addAttachmentsForExport() throws ImportExportException
    {
        Collection allAttachments = null;

        try
        {
            allAttachments = attachmentManager.getAllAttachments();
        }
        catch (ProviderException e)
        {
            throw new ImportExportException(e);
        }

        for (Iterator allPagesIter = allAttachments.iterator(); allPagesIter.hasNext();)
        {
            com.ecyrd.jspwiki.attachment.Attachment wikiAttachment = (com.ecyrd.jspwiki.attachment.Attachment) allPagesIter.next();

            try
            {
                Attachment attachment = createAttachment(wikiAttachment);
                Page page = (Page) pagesMap.get(wikiAttachment.getParentName());
                if (page != null)
                {
                    page.addAttachment(attachment);
                    attachments.add(attachment);

                    InputStream in = attachmentManager.getAttachmentStream(wikiAttachment);
                    FileUtils.copyFile(in, new File(new File(baseExportPath, page.getIdAsString()), attachment.getFileName()));
                }
            }
            catch (Exception e)
            {
                e.printStackTrace();
                System.out.println("exporting attachment failed: " + wikiAttachment.getFileName());
            }
        }
    }

    protected void backupEverything(String baseExportPath) throws ImportExportException
    {
        this.baseExportPath = getDestinationAttachmentsDirForCopying(baseExportPath);

        super.backupEverything(baseExportPath);
    }

    private Attachment createAttachment(com.ecyrd.jspwiki.attachment.Attachment wikiAttachment)
    {
        Attachment attachment = new Attachment();
        setCreationModificationInfo(attachment);
        attachment.setFileName(wikiAttachment.getFileName());
        attachment.setFileSize(wikiAttachment.getSize());
        attachment.setContentType("application/unknown");

        return attachment;
    }

    private Page createPage(WikiPage wikiPage) throws Exception
    {
        Page page = new Page();
        setCreationModificationInfo(page);
        page.setTitle(wikiPage.getName());
        String pageText = pageManager.getPageText(wikiPage.getName(), WikiPageProvider.LATEST_VERSION);
        page.setContent(clean(pageText));

        return page;
    }

    private String clean(String pageText)
    {
        return contentCleaner.clean(pageText);
    }

    private boolean unExportableWikiPage(WikiPage wikiPage)
    {
        String name = wikiPage.getName();
        return name.equalsIgnoreCase("LeftMenu");
    }

    protected List getSourceAttachmentsDirForCopying()
    {
        //we already copied all files to attachments folder of the export! Just return an empty list
        return new ArrayList();
    }
}
