package com.atlassian.confluence.jspwiki;

import com.atlassian.confluence.util.RegExpProcessor;
import com.opensymphony.util.TextUtils;

import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class ContentCleaner
{
    private List substitutions = new java.util.ArrayList();
    public static final String VAR_APPLICATION_NAME = "jspwiki.applicationName";
    public static final String VAR_ENCODING = "jspwiki.encoding";
    public static final String VAR_JSPWIKI_VERSION = "jspwikiversion";
    public static final String VAR_PAGE_NAME = "pagename";
    public static final String VAR_TOTAL_PAGES = "totalpages";

    public ContentCleaner(Properties variables)
    {
        // Substitutions are ordered, don't rearrange without good reason
        substitutions.add(new RegExpProcessor("__(.*?)__", "*$1*")); //bold
        substitutions.add(new RegExpProcessor("''(.*?)''", "_$1_")); //italic

        //headings
        substitutions.add(new RegExpProcessor("^!!!(.*?)$", "h2. $1")); //large heading
        substitutions.add(new RegExpProcessor("^!!(.*?)$", "h3. $1")); //medium heading
        substitutions.add(new RegExpProcessor("^!(.*?)$", "h4. $1")); //small heading

        substitutions.add(new RegExpProcessor("\\[([^\\[\\]\\p{Space}][^\\[\\]]+)(\\.png|\\.jpg|\\.gif)\\]", "!$1$2!")); //image

        //[page/bla.txt]
        substitutions.add(new RegExpProcessor("", "")
        {
            Pattern pattern = Pattern.compile("\\[(([^:\\[\\]\\p{Space}][^\\[\\]:]+)/)(.*?\\..*?)\\]");

            public String process(String input)
            {
                Matcher matcher = pattern.matcher(input);
                StringBuffer sb = new StringBuffer();
                while (matcher.find())
                {
                    // leave file:\\ links alone
                    if (matcher.group(2).startsWith("file:\\") || matcher.group(2).startsWith("file://") || matcher.group(2).startsWith("^"))
                        continue;

                    String replacementString = "[" + matcher.group(2) + "^" + matcher.group(3) + "]";
                    matcher.appendReplacement(sb, replacementString);
                }
                matcher.appendTail(sb);
                input = sb.toString();
                return super.process(input);
            }
        });

        //[bla.txt]
        substitutions.add(new RegExpProcessor("", "")
        {
            Pattern pattern = Pattern.compile("\\[([^:\\[\\]\\p{Space}\\{\\^][^\\[\\]\\|:\\^]+)(\\..*?)\\]");

            public String process(String input)
            {
                Matcher matcher = pattern.matcher(input);
                StringBuffer sb = new StringBuffer();
                while (matcher.find())
                {
                    // leave file:\\ links alone
                    if (matcher.group(1).startsWith("file:\\") || matcher.group(2).startsWith("file://"))
                        continue;

                    String replacementString = "[^" + matcher.group(1) + matcher.group(2) + "]";
                    matcher.appendReplacement(sb, replacementString);
                }
                matcher.appendTail(sb);
                input = sb.toString();
                return super.process(input);
            }
        });

        //most important variables
        substitutions.add(new RegExpProcessor("\\[\\{\\$applicationname\\}\\]", variables.getProperty(VAR_APPLICATION_NAME, "JSPWiki")));
        //substitutions.add(new RegExpProcessor("\\[\\{\\$baseurl\\}\\]", variables.getProperty("baseurl")));
        substitutions.add(new RegExpProcessor("\\[\\{\\$encoding\\}\\]", variables.getProperty(VAR_ENCODING, "UTF-8")));
        substitutions.add(new RegExpProcessor("\\[\\{\\$jspwikiversion\\}\\]", variables.getProperty(VAR_JSPWIKI_VERSION, "2.0.52")));
        substitutions.add(new RegExpProcessor("\\[\\{\\$pagename\\}\\]", variables.getProperty(VAR_PAGE_NAME, "This page")));
        substitutions.add(new RegExpProcessor("\\[\\{\\$totalpages\\}\\]", variables.getProperty(VAR_TOTAL_PAGES, "unknown")));

        // add space/indent between bullet and following text
        substitutions.add(new RegExpProcessor("^([\\p{Space}]*?)[\\*]([^\\p{Space}]*)$", "$1* $2"));

        // handles pages links (also removes spaces from page link names. eg. [Iteration 12] --> [Iteration12] and
        // [Alias Iteration 12|Iteration 12] --> [Alias Iteration 12|Iteration12]
        substitutions.add(new RegExpProcessor("", "")
        {
            Pattern pattern = Pattern.compile("\\[([^\\]\\|\\$\\{\\^]*)\\|?([^\\]\\$\\{]*)\\]");

            public String process(String input)
            {
                Matcher matcher = pattern.matcher(input);
                StringBuffer sb = new StringBuffer();
                while (matcher.find())
                {
                    // leave file:\\ links alone
                    String linkBody = matcher.group(2);
                    if (matcher.group(1).startsWith("file:\\") || linkBody.startsWith("file://") || linkBody.startsWith("^"))
                        continue;

                    String replacementString = "";
                    if (TextUtils.stringSet(linkBody))
                    {
                        replacementString = "[" + matcher.group(1) + "|" + processLink(linkBody.replaceAll(" ", "")) + "]";
                    }
                    else
                    {
                        replacementString = "[" + processLink(matcher.group(1)) + "]";
                    }

                    matcher.appendReplacement(sb, replacementString);
                }
                matcher.appendTail(sb);
                input = sb.toString();
                return super.process(input);
            }
        });

        //what to do with these?
        //[1]        = Makes a reference to a footnote numbered 1.
        //[#1]       = Marks the footnote number 1.
        //;term:ex   = makes a definition for 'term' with the explanation 'ex'
    }

    public String clean(String input)
    {
        List noformatBlocks = new ArrayList();

        // replace all {{{ ... }}} with {{{replaced}}} and add blocks to a list (so that they don't get processed)
        // we'll add these blocks back afterwards
        Pattern pattern = Pattern.compile("\\{\\{\\{(.*)\\}\\}\\}", Pattern.DOTALL | Pattern.MULTILINE);
        Matcher matcher = pattern.matcher(input);
        StringBuffer sb = new StringBuffer();
        while (matcher.find())
        {
            noformatBlocks.add(matcher.group(1));
            matcher.appendReplacement(sb, "{{{replaced}}}");
        }
        matcher.appendTail(sb);
        input = sb.toString();

        for (Iterator it = substitutions.iterator(); it.hasNext();)
        {
            input = ((RegExpProcessor) it.next()).process(input);
        }

        // now put those blocks back (that is {{{replaced}}} --> {noformat}original block{noformat})
        pattern = Pattern.compile("\\{\\{\\{replaced\\}\\}\\}", Pattern.DOTALL | Pattern.MULTILINE);
        matcher = pattern.matcher(input);
        sb = new StringBuffer();
        int i = 0;
        while (matcher.find())
        {
            matcher.appendReplacement(sb, "{noformat}" + escapeSpecialRegexChars((String) noformatBlocks.get(i++)) + "{noformat}");
        }
        matcher.appendTail(sb);
        input = sb.toString();

        return input;
    }

    // escape backslashes "\" and dollar signs "$"
    private String escapeSpecialRegexChars(String input)
    {
        StringBuffer sb = new StringBuffer();
        for (int i = 0; i < input.toCharArray().length; i++)
        {
            char c = input.toCharArray()[i];
            if (c == '\\' || c == '$')
            {
                sb.append('\\');
            }
            sb.append(c);
        }
        return sb.toString();
    }

    private String processLink(String str)
    {
        // replace all spaces
        str = str.replaceAll(" ", "");

        // if there are hyphens, remove and camel case the title
        if (str.indexOf("-") != -1)
        {
            String result = "";

            StringTokenizer st = new StringTokenizer(str, "-");
            while (st.hasMoreTokens())
            {
                String token = st.nextToken();
                result += upFirstLetter(token);
            }

            str = result;
        }

        return str;
    }

    private String upFirstLetter(String str)
    {
        if (str.length() < 2)
            return str;

        return str.substring(0,1).toUpperCase() + str.substring(1);
    }
}
